@@ -1,6 +1,5 @@
 /* Internal interfaces for the QNX Neutrino specific target code for gdbserver.
-   Copyright (C) 2009
-   Free Software Foundation, Inc.
+   Copyright (C) 2009-2019 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,8 +16,10 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef NTO_LOW_H
-#define NTO_LOW_H
+#ifndef GDBSERVER_NTO_LOW_H
+#define GDBSERVER_NTO_LOW_H
+
+struct target_desc;
 
 enum regset_type
 {
@@ -41,5 +42,8 @@ struct nto_target_ops
 
 extern struct nto_target_ops the_low_target;
 
-#endif
+/* The inferior's target description.  This is a global because the
+   LynxOS ports support neither bi-arch nor multi-process.  */
+extern const struct target_desc *nto_tdesc;
 
+#endif /* GDBSERVER_NTO_LOW_H */
