@@ -95,10 +95,10 @@ int yyparse (void);
 
 static int yylex (void);
 
-void yyerror (const char *);
+static void yyerror (const char *);
 
 
-#line 102 "go-exp.c" /* yacc.c:339  */
+#line 102 "go-exp.c.tmp" /* yacc.c:339  */
 
 # ifndef YY_NULLPTRPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -245,7 +245,7 @@ union YYSTYPE
     struct stoken_vector svec;
   
 
-#line 249 "go-exp.c" /* yacc.c:355  */
+#line 249 "go-exp.c.tmp" /* yacc.c:355  */
 };
 
 typedef union YYSTYPE YYSTYPE;
@@ -267,7 +267,7 @@ int yyparse (void);
 static int parse_number (struct parser_state *,
 			 const char *, int, int, YYSTYPE *);
 
-#line 271 "go-exp.c" /* yacc.c:358  */
+#line 271 "go-exp.c.tmp" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -573,7 +573,7 @@ static const yytype_uint16 yyrline[] =
      321,   325,   329,   333,   337,   341,   345,   349,   353,   357,
      361,   365,   369,   373,   377,   381,   385,   389,   393,   397,
      401,   407,   414,   423,   436,   443,   446,   452,   467,   474,
-     491,   509,   521,   527,   533,   549,   601,   603,   610,   623
+     491,   509,   521,   527,   533,   549,   596,   598,   605,   618
 };
 #endif
 
@@ -1500,61 +1500,61 @@ yyreduce:
     { write_exp_elt_opcode (pstate, OP_TYPE);
 			  write_exp_elt_type (pstate, (yyvsp[0].tval));
 			  write_exp_elt_opcode (pstate, OP_TYPE); }
-#line 1505 "go-exp.c" /* yacc.c:1646  */
+#line 1505 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 6:
 #line 204 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_COMMA); }
-#line 1511 "go-exp.c" /* yacc.c:1646  */
+#line 1511 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 7:
 #line 209 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_IND); }
-#line 1517 "go-exp.c" /* yacc.c:1646  */
+#line 1517 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 8:
 #line 213 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_ADDR); }
-#line 1523 "go-exp.c" /* yacc.c:1646  */
+#line 1523 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 9:
 #line 217 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_NEG); }
-#line 1529 "go-exp.c" /* yacc.c:1646  */
+#line 1529 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 10:
 #line 221 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_PLUS); }
-#line 1535 "go-exp.c" /* yacc.c:1646  */
+#line 1535 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 11:
 #line 225 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_LOGICAL_NOT); }
-#line 1541 "go-exp.c" /* yacc.c:1646  */
+#line 1541 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 12:
 #line 229 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_COMPLEMENT); }
-#line 1547 "go-exp.c" /* yacc.c:1646  */
+#line 1547 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 13:
 #line 233 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_POSTINCREMENT); }
-#line 1553 "go-exp.c" /* yacc.c:1646  */
+#line 1553 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 14:
 #line 237 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_POSTDECREMENT); }
-#line 1559 "go-exp.c" /* yacc.c:1646  */
+#line 1559 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 15:
@@ -1562,7 +1562,7 @@ yyreduce:
     { write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
 			  write_exp_string (pstate, (yyvsp[0].ssym).stoken);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
-#line 1567 "go-exp.c" /* yacc.c:1646  */
+#line 1567 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 16:
@@ -1571,7 +1571,7 @@ yyreduce:
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
 			  write_exp_string (pstate, (yyvsp[-1].ssym).stoken);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
-#line 1576 "go-exp.c" /* yacc.c:1646  */
+#line 1576 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 17:
@@ -1583,19 +1583,19 @@ yyreduce:
 			  s.length = 0;
 			  write_exp_string (pstate, s);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
-#line 1588 "go-exp.c" /* yacc.c:1646  */
+#line 1588 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 18:
 #line 266 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_SUBSCRIPT); }
-#line 1594 "go-exp.c" /* yacc.c:1646  */
+#line 1594 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 19:
 #line 272 "go-exp.y" /* yacc.c:1646  */
     { start_arglist (); }
-#line 1600 "go-exp.c" /* yacc.c:1646  */
+#line 1600 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 20:
@@ -1604,31 +1604,31 @@ yyreduce:
 			  write_exp_elt_longcst (pstate,
 						 (LONGEST) end_arglist ());
 			  write_exp_elt_opcode (pstate, OP_FUNCALL); }
-#line 1609 "go-exp.c" /* yacc.c:1646  */
+#line 1609 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 21:
 #line 281 "go-exp.y" /* yacc.c:1646  */
     { start_arglist (); }
-#line 1615 "go-exp.c" /* yacc.c:1646  */
+#line 1615 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 23:
 #line 288 "go-exp.y" /* yacc.c:1646  */
     { arglist_len = 1; }
-#line 1621 "go-exp.c" /* yacc.c:1646  */
+#line 1621 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 24:
 #line 292 "go-exp.y" /* yacc.c:1646  */
     { arglist_len++; }
-#line 1627 "go-exp.c" /* yacc.c:1646  */
+#line 1627 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 25:
 #line 296 "go-exp.y" /* yacc.c:1646  */
     { (yyval.lval) = end_arglist () - 1; }
-#line 1633 "go-exp.c" /* yacc.c:1646  */
+#line 1633 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 26:
@@ -1636,7 +1636,7 @@ yyreduce:
     { write_exp_elt_opcode (pstate, UNOP_MEMVAL);
 			  write_exp_elt_type (pstate, (yyvsp[-2].tval));
 			  write_exp_elt_opcode (pstate, UNOP_MEMVAL); }
-#line 1641 "go-exp.c" /* yacc.c:1646  */
+#line 1641 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 27:
@@ -1644,139 +1644,139 @@ yyreduce:
     { write_exp_elt_opcode (pstate, UNOP_CAST);
 			  write_exp_elt_type (pstate, (yyvsp[-3].tval));
 			  write_exp_elt_opcode (pstate, UNOP_CAST); }
-#line 1649 "go-exp.c" /* yacc.c:1646  */
+#line 1649 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 28:
 #line 312 "go-exp.y" /* yacc.c:1646  */
     { }
-#line 1655 "go-exp.c" /* yacc.c:1646  */
+#line 1655 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 29:
 #line 318 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_REPEAT); }
-#line 1661 "go-exp.c" /* yacc.c:1646  */
+#line 1661 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 30:
 #line 322 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_MUL); }
-#line 1667 "go-exp.c" /* yacc.c:1646  */
+#line 1667 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 31:
 #line 326 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_DIV); }
-#line 1673 "go-exp.c" /* yacc.c:1646  */
+#line 1673 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 32:
 #line 330 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_REM); }
-#line 1679 "go-exp.c" /* yacc.c:1646  */
+#line 1679 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 33:
 #line 334 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_ADD); }
-#line 1685 "go-exp.c" /* yacc.c:1646  */
+#line 1685 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 34:
 #line 338 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_SUB); }
-#line 1691 "go-exp.c" /* yacc.c:1646  */
+#line 1691 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 35:
 #line 342 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LSH); }
-#line 1697 "go-exp.c" /* yacc.c:1646  */
+#line 1697 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 36:
 #line 346 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_RSH); }
-#line 1703 "go-exp.c" /* yacc.c:1646  */
+#line 1703 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 37:
 #line 350 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_EQUAL); }
-#line 1709 "go-exp.c" /* yacc.c:1646  */
+#line 1709 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 38:
 #line 354 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_NOTEQUAL); }
-#line 1715 "go-exp.c" /* yacc.c:1646  */
+#line 1715 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 39:
 #line 358 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LEQ); }
-#line 1721 "go-exp.c" /* yacc.c:1646  */
+#line 1721 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 40:
 #line 362 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_GEQ); }
-#line 1727 "go-exp.c" /* yacc.c:1646  */
+#line 1727 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 41:
 #line 366 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LESS); }
-#line 1733 "go-exp.c" /* yacc.c:1646  */
+#line 1733 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 42:
 #line 370 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_GTR); }
-#line 1739 "go-exp.c" /* yacc.c:1646  */
+#line 1739 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 43:
 #line 374 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_AND); }
-#line 1745 "go-exp.c" /* yacc.c:1646  */
+#line 1745 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 44:
 #line 378 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_XOR); }
-#line 1751 "go-exp.c" /* yacc.c:1646  */
+#line 1751 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 45:
 #line 382 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_IOR); }
-#line 1757 "go-exp.c" /* yacc.c:1646  */
+#line 1757 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 46:
 #line 386 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LOGICAL_AND); }
-#line 1763 "go-exp.c" /* yacc.c:1646  */
+#line 1763 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 47:
 #line 390 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LOGICAL_OR); }
-#line 1769 "go-exp.c" /* yacc.c:1646  */
+#line 1769 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 48:
 #line 394 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, TERNOP_COND); }
-#line 1775 "go-exp.c" /* yacc.c:1646  */
+#line 1775 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 49:
 #line 398 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_ASSIGN); }
-#line 1781 "go-exp.c" /* yacc.c:1646  */
+#line 1781 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 50:
@@ -1784,7 +1784,7 @@ yyreduce:
     { write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY);
 			  write_exp_elt_opcode (pstate, (yyvsp[-1].opcode));
 			  write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY); }
-#line 1789 "go-exp.c" /* yacc.c:1646  */
+#line 1789 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 51:
@@ -1793,7 +1793,7 @@ yyreduce:
 			  write_exp_elt_type (pstate, (yyvsp[0].typed_val_int).type);
 			  write_exp_elt_longcst (pstate, (LONGEST)((yyvsp[0].typed_val_int).val));
 			  write_exp_elt_opcode (pstate, OP_LONG); }
-#line 1798 "go-exp.c" /* yacc.c:1646  */
+#line 1798 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 52:
@@ -1804,7 +1804,7 @@ yyreduce:
 			  vec.tokens = &(yyvsp[0].tsval);
 			  write_exp_string_vector (pstate, (yyvsp[0].tsval).type, &vec);
 			}
-#line 1809 "go-exp.c" /* yacc.c:1646  */
+#line 1809 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 53:
@@ -1818,7 +1818,7 @@ yyreduce:
 						 val.typed_val_int.val);
 			  write_exp_elt_opcode (pstate, OP_LONG);
 			}
-#line 1823 "go-exp.c" /* yacc.c:1646  */
+#line 1823 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 54:
@@ -1827,7 +1827,7 @@ yyreduce:
 			  write_exp_elt_type (pstate, (yyvsp[0].typed_val_float).type);
 			  write_exp_elt_floatcst (pstate, (yyvsp[0].typed_val_float).val);
 			  write_exp_elt_opcode (pstate, OP_FLOAT); }
-#line 1832 "go-exp.c" /* yacc.c:1646  */
+#line 1832 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 56:
@@ -1835,7 +1835,7 @@ yyreduce:
     {
 			  write_dollar_variable (pstate, (yyvsp[0].sval));
 			}
-#line 1840 "go-exp.c" /* yacc.c:1646  */
+#line 1840 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 57:
@@ -1852,7 +1852,7 @@ yyreduce:
 						 (LONGEST) TYPE_LENGTH ((yyvsp[-1].tval)));
 			  write_exp_elt_opcode (pstate, OP_LONG);
 			}
-#line 1857 "go-exp.c" /* yacc.c:1646  */
+#line 1857 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 58:
@@ -1861,7 +1861,7 @@ yyreduce:
 			  /* TODO(dje): Go objects in structs.  */
 			  write_exp_elt_opcode (pstate, UNOP_SIZEOF);
 			}
-#line 1866 "go-exp.c" /* yacc.c:1646  */
+#line 1866 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 59:
@@ -1881,7 +1881,7 @@ yyreduce:
 			  vec->ptr = (char *) xmalloc ((yyvsp[0].tsval).length + 1);
 			  memcpy (vec->ptr, (yyvsp[0].tsval).ptr, (yyvsp[0].tsval).length + 1);
 			}
-#line 1886 "go-exp.c" /* yacc.c:1646  */
+#line 1886 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 60:
@@ -1901,7 +1901,7 @@ yyreduce:
 			  (yyval.svec).tokens[(yyval.svec).len - 1].length = (yyvsp[0].tsval).length;
 			  (yyval.svec).tokens[(yyval.svec).len - 1].ptr = p;
 			}
-#line 1906 "go-exp.c" /* yacc.c:1646  */
+#line 1906 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 61:
@@ -1915,7 +1915,7 @@ yyreduce:
 			    xfree ((yyvsp[0].svec).tokens[i].ptr);
 			  xfree ((yyvsp[0].svec).tokens);
 			}
-#line 1920 "go-exp.c" /* yacc.c:1646  */
+#line 1920 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 62:
@@ -1923,7 +1923,7 @@ yyreduce:
     { write_exp_elt_opcode (pstate, OP_BOOL);
 			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[0].lval));
 			  write_exp_elt_opcode (pstate, OP_BOOL); }
-#line 1928 "go-exp.c" /* yacc.c:1646  */
+#line 1928 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 63:
@@ -1931,7 +1931,7 @@ yyreduce:
     { write_exp_elt_opcode (pstate, OP_BOOL);
 			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[0].lval));
 			  write_exp_elt_opcode (pstate, OP_BOOL); }
-#line 1936 "go-exp.c" /* yacc.c:1646  */
+#line 1936 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 64:
@@ -1949,7 +1949,7 @@ yyreduce:
 			  write_exp_elt_sym (pstate, sym);
 			  write_exp_elt_opcode (pstate, OP_VAR_ENTRY_VALUE);
 			}
-#line 1954 "go-exp.c" /* yacc.c:1646  */
+#line 1954 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 65:
@@ -1959,12 +1959,7 @@ yyreduce:
 			  if (sym.symbol)
 			    {
 			      if (symbol_read_needs_frame (sym.symbol))
-				{
-				  if (innermost_block == 0
-				      || contained_in (sym.block,
-						       innermost_block))
-				    innermost_block = sym.block;
-				}
+				innermost_block.update (sym);
 
 			      write_exp_elt_opcode (pstate, OP_VAR_VALUE);
 			      write_exp_elt_block (pstate, sym.block);
@@ -1995,30 +1990,30 @@ yyreduce:
 				       copy_name ((yyvsp[0].ssym).stoken));
 			    }
 			}
-#line 2000 "go-exp.c" /* yacc.c:1646  */
+#line 1995 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 66:
-#line 602 "go-exp.y" /* yacc.c:1646  */
+#line 597 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = lookup_pointer_type ((yyvsp[0].tval)); }
-#line 2006 "go-exp.c" /* yacc.c:1646  */
+#line 2001 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 67:
-#line 604 "go-exp.y" /* yacc.c:1646  */
+#line 599 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = (yyvsp[0].tsym).type; }
-#line 2012 "go-exp.c" /* yacc.c:1646  */
+#line 2007 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 68:
-#line 611 "go-exp.y" /* yacc.c:1646  */
+#line 606 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = builtin_go_type (parse_gdbarch (pstate))
 			    ->builtin_uint8; }
-#line 2019 "go-exp.c" /* yacc.c:1646  */
+#line 2014 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
 
-#line 2023 "go-exp.c" /* yacc.c:1646  */
+#line 2018 "go-exp.c.tmp" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2246,7 +2241,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 633 "go-exp.y" /* yacc.c:1906  */
+#line 628 "go-exp.y" /* yacc.c:1906  */
 
 
 /* Take care of parsing a number (anything that starts with a digit).
@@ -2710,7 +2705,7 @@ lex_one_token (struct parser_state *par_state)
 	    last_was_structop = 1;
 	  goto symbol;		/* Nope, must be a symbol. */
 	}
-      /* FALL THRU into number case.  */
+      /* FALL THRU.  */
 
     case '0':
     case '1':
@@ -2901,17 +2896,15 @@ lex_one_token (struct parser_state *par_state)
 }
 
 /* An object of this type is pushed on a FIFO by the "outer" lexer.  */
-typedef struct
+struct token_and_value
 {
   int token;
   YYSTYPE value;
-} token_and_value;
-
-DEF_VEC_O (token_and_value);
+};
 
 /* A FIFO of tokens that have been read but not yet returned to the
    parser.  */
-static VEC (token_and_value) *token_fifo;
+static std::vector<token_and_value> token_fifo;
 
 /* Non-zero if the lexer should return tokens from the FIFO.  */
 static int popping;
@@ -3107,10 +3100,10 @@ yylex (void)
 {
   token_and_value current, next;
 
-  if (popping && !VEC_empty (token_and_value, token_fifo))
+  if (popping && !token_fifo.empty ())
     {
-      token_and_value tv = *VEC_index (token_and_value, token_fifo, 0);
-      VEC_ordered_remove (token_and_value, token_fifo, 0);
+      token_and_value tv = token_fifo[0];
+      token_fifo.erase (token_fifo.begin ());
       yylval = tv.value;
       /* There's no need to fall through to handle package.name
 	 as that can never happen here.  In theory.  */
@@ -3163,13 +3156,11 @@ yylex (void)
 	    }
 	}
 
-      VEC_safe_push (token_and_value, token_fifo, &next);
-      VEC_safe_push (token_and_value, token_fifo, &name2);
+      token_fifo.push_back (next);
+      token_fifo.push_back (name2);
     }
   else
-    {
-      VEC_safe_push (token_and_value, token_fifo, &next);
-    }
+    token_fifo.push_back (next);
 
   /* If we arrive here we don't have a package-qualified name.  */
 
@@ -3193,18 +3184,18 @@ go_parse (struct parser_state *par_state)
   last_was_structop = 0;
   saw_name_at_eof = 0;
 
-  VEC_free (token_and_value, token_fifo);
+  token_fifo.clear ();
   popping = 0;
   name_obstack.clear ();
 
   return yyparse ();
 }
 
-void
+static void
 yyerror (const char *msg)
 {
   if (prev_lexptr)
     lexptr = prev_lexptr;
 
-  error (_("A %s in expression, near `%s'."), (msg ? msg : "error"), lexptr);
+  error (_("A %s in expression, near `%s'."), msg, lexptr);
 }
