@@ -1,6 +1,6 @@
 /* Target-dependent code for the Matsushita MN10300 for GDB, the GNU debugger.
 
-   Copyright (C) 2003-2018 Free Software Foundation, Inc.
+   Copyright (C) 2003-2019 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -472,13 +472,13 @@ static const struct tramp_frame am33_linux_sigframe = {
   1,
   {
     /* mov     119,d0 */
-    { 0x2c, -1 },
-    { 0x77, -1 },
-    { 0x00, -1 },
+    { 0x2c, ULONGEST_MAX },
+    { 0x77, ULONGEST_MAX },
+    { 0x00, ULONGEST_MAX },
     /* syscall 0 */
-    { 0xf0, -1 },
-    { 0xe0, -1 },
-    { TRAMP_SENTINEL_INSN, -1 }
+    { 0xf0, ULONGEST_MAX },
+    { 0xe0, ULONGEST_MAX },
+    { TRAMP_SENTINEL_INSN, ULONGEST_MAX }
   },
   am33_linux_sigframe_cache_init
 };
@@ -488,13 +488,13 @@ static const struct tramp_frame am33_linux_rt_sigframe = {
   1,
   {
     /* mov     173,d0 */
-    { 0x2c, -1 },
-    { 0xad, -1 },
-    { 0x00, -1 },
+    { 0x2c, ULONGEST_MAX },
+    { 0xad, ULONGEST_MAX },
+    { 0x00, ULONGEST_MAX },
     /* syscall 0 */
-    { 0xf0, -1 },
-    { 0xe0, -1 },
-    { TRAMP_SENTINEL_INSN, -1 }
+    { 0xf0, ULONGEST_MAX },
+    { 0xe0, ULONGEST_MAX },
+    { TRAMP_SENTINEL_INSN, ULONGEST_MAX }
   },
   am33_linux_sigframe_cache_init
 };
